//-----------------------------------------------------------------------------
// Title      : bsp_ub040_mk3_ext_eth3
//-----------------------------------------------------------------------------
// Funktion   : Externes Modul : Ethernet-Modul mit ENC28J60
// Schaltung  : myAVR MK3 + ETH-Modul
//-----------------------------------------------------------------------------
// Prozessor  : ATMega2560
// Takt       : 16 MHz
// Sprache    : C
// Date       : 16.12.2011
// Version    : 1.0
// Autor      : UB
//-----------------------------------------------------------------------------
#define F_CPU 16000000
#include <avr/io.h>
#include <ub_mk3_2560/MK3_2560_LCD.h>
#include <ub_mk3_2560/MK3_2560_KEY.h>
#include <ub_ext_module/EXT_MOD_ETH.h>


//-----------------------------------------------------------------------------
// Hinweis : Quick-Jumper Port-C+A und Port-K muss gesetzt sein
// - das Ethernet-Modul muss angeschlossen sein
// - das Programm zeigt die HOST IP-Adresse an
// - mit dem Button-1 kann eine ARP-Request an
//   den HOST gesendet werden, der HOST muss mit ARP-Reply antworten
// - es wird angezeigt ob der HOST geantwortet hat oder nicht
// - die Anzahl der gesendeten und empfangenen Packete
//   wird auch angezeigt
//-----------------------------------------------------------------------------
int main (void)
{
    unsigned int wert=0;
    
    // Init der Tasten
    MK3_INIT_KEY();
    
    // Init vo m Display
	MK3_INIT_LCD();					// init des LCD
	MK3_LCD_CLEAR();				// LCD loeschen
	MK3_LCD_LIGHT(true);			// beleuchtung einschalten
	
    MK3_LCD_STRING_AT(0,0,"ETH-Modul Test:03");
    
    // Init vom ENC28J60
    EXT_INIT_ETH();
    
    if(ENC28J60.status_ok==true) {
      // starten vom Ethernet-Modul
      EXT_ETH_START();
      
      MK3_LCD_STRING_AT(0,8,"ENC28J60 OK");
      
      if(USE_ETH_INTMODE==1) {
        MK3_LCD_STRING_AT(0,16,"Interrupt-Mode");
      }
      else {
        MK3_LCD_STRING_AT(0,16,"Polling-Mode");
      }
      MK3_LCD_STRING_AT(0,32,"HOST=");
      MK3_LCD_PRINT_UTYP(HOST_ETH_Data.host_ip[0],DEZ_U0);
      MK3_LCD_PRINT_STRING(".");
      MK3_LCD_PRINT_UTYP(HOST_ETH_Data.host_ip[1],DEZ_U0);
      MK3_LCD_PRINT_STRING(".");
      MK3_LCD_PRINT_UTYP(HOST_ETH_Data.host_ip[2],DEZ_U0);
      MK3_LCD_PRINT_STRING(".");
      MK3_LCD_PRINT_UTYP(HOST_ETH_Data.host_ip[3],DEZ_U0);
      MK3_LCD_STRING_AT(0,40,"Button-1=check HOST");
    }
    else {
      MK3_LCD_STRING_AT(0,16,"ENC28J60 Error");
      MK3_LCD_STRING_AT(0,24,"> Hardware pruefen !");
      MK3_LCD_STRING_AT(0,56,"Modul ohne Funktion");
    }
    
    
	while (true)
	{
	  if(ENC28J60.status_ok==true) {
	    // wenn alles OK
        if(USE_ETH_INTMODE!=1) {
          // im Polling-Betrieb
          EXT_ETH_POLL();
        }
        
        if(MK3_KEY_SET(BUTTON1)==true) {
          // wenn Button-1 gedrueckt wurde
          MK3_LCD_STRING_AT(0,48,"                   ");
          if(EXT_ETH_CHECK_HOST()==true) {
            MK3_LCD_STRING_AT(0,48,"HOST OK");
          }
          else {
            MK3_LCD_STRING_AT(0,48,"keine Antwort");
          }
        }
        
        // Anzahl der TX und RX Packete anzeigen
        wert=ENC28J60.tx_packete;
        MK3_LCD_STRING_AT(0,56,"TX=");
        MK3_LCD_PRINT_UTYP(wert,DEZ_U0);
        wert=ENC28J60.rx_packete;
        MK3_LCD_STRING_AT(45,56,"RX=");
        MK3_LCD_PRINT_UTYP(wert,DEZ_U0);
        wert=ENC28J60.rx_unknonwn;
        MK3_LCD_STRING_AT(90,56,"?=");
        MK3_LCD_PRINT_UTYP(wert,DEZ_U0);
      }
	}
	return 0;
}
//-----------------------------------------------------------------------------
