//-----------------------------------------------------------------------------
// Title      : bsp_ub041_mk3_ext_eth4
//-----------------------------------------------------------------------------
// Funktion   : Externes Modul : Ethernet-Modul mit ENC28J60
// Schaltung  : myAVR MK3 + ETH-Modul
//-----------------------------------------------------------------------------
// Prozessor  : ATMega2560
// Takt       : 16 MHz
// Sprache    : C
// Date       : 17.12.2011
// Version    : 1.0
// Autor      : UB
//-----------------------------------------------------------------------------
#define F_CPU 16000000
#include <avr/io.h>
#include <ub_mk3_2560/MK3_2560_LCD.h>
#include <ub_mk3_2560/MK3_2560_KEY.h>
#include <ub_ext_module/EXT_MOD_ETH.h>


//-----------------------------------------------------------------------------
// Hinweis : Quick-Jumper Port-C+A und Port-K muss gesetzt sein
// - das Ethernet-Modul muss angeschlossen sein
// - das Programm zeigt die eigen IP-Adresse und den UDP-Port an
// - mit dem Button-1 kann ein String per UDP an
//   den HOST gesendet werden
//   (zum Test ob der HOST da ist, wird vorher eine ARP-Request gesendet)
// - es wird angezeigt ob der HOST da ist oder nicht
// - der HOST kann Strings per UDP an die CPU sende
//   diese werden dann angezeigt
// - die Anzahl der gesendeten und empfangenen Packete
//   wird auch angezeigt
//-----------------------------------------------------------------------------
int main (void)
{
    unsigned int wert=0;
    char s_buf[UDP_DATA_LEN];
    
    // Init der Tasten
    MK3_INIT_KEY();
    
    // Init vo m Display
	MK3_INIT_LCD();					// init des LCD
	MK3_LCD_CLEAR();				// LCD loeschen
	MK3_LCD_LIGHT(true);			// beleuchtung einschalten
	
    MK3_LCD_STRING_AT(0,0,"ETH-Modul Test:04");
    
    // Init vom ENC28J60
    EXT_INIT_ETH();
    
    if(ENC28J60.status_ok==true) {
      // starten vom Ethernet-Modul
      EXT_ETH_START();
      
      MK3_LCD_STRING_AT(0,8,"ENC28J60 OK");
      
      if(USE_ETH_INTMODE==1) {
        MK3_LCD_STRING_AT(75,8,"Int-Mode");
      }
      else {
        MK3_LCD_STRING_AT(75,8,"Pol-Mode");
      }
      MK3_LCD_STRING_AT(0,24,"myIP=");
      MK3_LCD_PRINT_UTYP(ETH_Setting.own_ip[0],DEZ_U0);
      MK3_LCD_PRINT_STRING(".");
      MK3_LCD_PRINT_UTYP(ETH_Setting.own_ip[1],DEZ_U0);
      MK3_LCD_PRINT_STRING(".");
      MK3_LCD_PRINT_UTYP(ETH_Setting.own_ip[2],DEZ_U0);
      MK3_LCD_PRINT_STRING(".");
      MK3_LCD_PRINT_UTYP(ETH_Setting.own_ip[3],DEZ_U0);
      MK3_LCD_STRING_AT(0,32,"UDP-Port=");
      MK3_LCD_PRINT_UTYP(ETH_Setting.own_udp_port,DEZ_U0);
      MK3_LCD_STRING_AT(0,40,"Button-1=send per UDP");
    }
    else {
      MK3_LCD_STRING_AT(0,16,"ENC28J60 Error");
      MK3_LCD_STRING_AT(0,24,"> Hardware pruefen !");
      MK3_LCD_STRING_AT(0,56,"Modul ohne Funktion");
    }
    
    
	while (true)
	{
	  if(ENC28J60.status_ok==true) {
	    // wenn alles OK
        if(USE_ETH_INTMODE!=1) {
          // im Polling-Betrieb
          EXT_ETH_POLL();
        }
        
        // check auf button
        if(MK3_KEY_SET(BUTTON1)==true) {
          // wenn Button-1 gedrueckt wurde
          MK3_LCD_STRING_AT(0,48,"                   ");
          if(EXT_ETH_CHECK_HOST()==true) {
            MK3_LCD_STRING_AT(0,48,"HOST OK      ");
            // String per UDP senden
            EXT_ETH_UDP_OUT("Hallo PC");
          }
          else {
            MK3_LCD_STRING_AT(0,48,"keine Antwort");
          }
        }
        
        // check auf empfangene UDP-Packete
        if(EXT_ETH_UDP_IN(s_buf)==true) {
          // wenn etwas empfangen wurde
          MK3_LCD_STRING_AT(0,48,"                   ");
          MK3_LCD_STRING_AT(0,48,"RX-UDP=");
          MK3_LCD_PRINT_STRING(s_buf);
        }
        
        // Anzahl der TX und RX Packete anzeigen
        wert=ENC28J60.tx_packete;
        MK3_LCD_STRING_AT(0,56,"TX=");
        MK3_LCD_PRINT_UTYP(wert,DEZ_U0);
        wert=ENC28J60.rx_packete;
        MK3_LCD_STRING_AT(45,56,"RX=");
        MK3_LCD_PRINT_UTYP(wert,DEZ_U0);
        wert=ENC28J60.rx_unknonwn;
        MK3_LCD_STRING_AT(90,56,"?=");
        MK3_LCD_PRINT_UTYP(wert,DEZ_U0);
      }
	}
	return 0;
}
//-----------------------------------------------------------------------------
