//-----------------------------------------------------------------------------
// ETH_LO_ENC28J60
// Version 3.0
// 17.12.2011 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Lo-Level Funktionen
// mit dem Ethernet-IC : ENC28J60
//
// Funktionen : Einbindung der ENC28J60 Grundfunktionen
//
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_EXT_MODULE/UB_ETH_LOLEVEL/"
// liegen und mit #include <UB_ETH_LOLEVEL/ETH_LO_ENC28J60.h>
// eingebunden werden
//-----------------------------------------------------------------------------


#ifndef UB_ETH_ENC    // Wichtig !!
#define UB_ETH_ENC 1  // damit wird das File nur einmal eingebunden

//----------------------------------------------------------------------------
// Includes
//----------------------------------------------------------------------------
// keine



//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//-----------------------------------------------------------------------------
// siehe ETH_LO_GLOBAL.h


//-----------------------------------------------------------------------------
// Funktionen die von extern benutzt werden koennen
//-----------------------------------------------------------------------------
unsigned char ENC_readControlRegisterE(unsigned char adr);
unsigned char ENC_readControlRegisterM(unsigned char adr);
void ENC_writeControlRegister(unsigned char adr,unsigned char wert);
void ENC_setControlRegisterBank(unsigned char bank);
void ENC_clearBitRegister(unsigned char adr,unsigned char wert);
void ENC_setBitRegister(unsigned char adr,unsigned char wert);
unsigned int ENC_readPhyRegister(unsigned char adr);
void ENC_writePhyRegister(unsigned char adr, unsigned int wert);
unsigned char ENC_readBufferMemory(void);
void ENC_readBufferBlock(unsigned char mode, unsigned int laenge);
void ENC_writeBufferMemory(unsigned char wert);
void ENC_writeBufferBlock(unsigned int laenge);
bool ENC_checkID(void);
unsigned char ENC_checkVersion(void);
void ENC_setLedMode(ETH_LED_MODE_t led_a, ETH_LED_MODE_t led_b);
void ENC_setBuffer(ETH_RAM_t aufteilung);
void ENC_softReset(void);
void ENC_setMAC(void);
void ENC_initENC28J60(void);
void ENC_rxEnable(void);
void ENC_IntEnable(void);
unsigned int ENC_getFreeRXPuffer(void);


//-----------------------------------------------------------------------------
// interne Defines  Control-Register
// CRE = ETH-Register                    -> ENC_readControlRegisterE
// CRM = MAC-Register oder MII-Register  -> ENC_readControlRegisterM
//-----------------------------------------------------------------------------
#define ENC28J60_CRE_ERDPTL   0x00  // ERDPTL   (Bank 0) Read Buffer Datapointer
#define ENC28J60_CRE_ERDPTH   0x01  // ERDPTL   (Bank 0)
#define ENC28J60_CRE_EWRPTL   0x02  // EWRPTL   (Bank 0) Write Buffer Datapointer
#define ENC28J60_CRE_EWRPTH   0x03  // EWRPTL   (Bank 0)
#define ENC28J60_CRE_ETXSTL   0x04  // ETXSTL   (Bank 0) TX Buffer Start
#define ENC28J60_CRE_ETXSTH   0x05  // ETXSTH   (Bank 0)
#define ENC28J60_CRE_ETXNDL   0x06  // ETXNDL   (Bank 0) TX Buffer Ende
#define ENC28J60_CRE_ETXNDH   0x07  // ETXNDH   (Bank 0)
#define ENC28J60_CRE_ERXSTL   0x08  // ERXSTL   (Bank 0) RX Buffer Start
#define ENC28J60_CRE_ERXSTH   0x09  // ERXSTH   (Bank 0)
#define ENC28J60_CRE_ERXNDL   0x0A  // ERXNDL   (Bank 0) RX Buffer Ende
#define ENC28J60_CRE_ERXNDH   0x0B  // ERXNDH   (Bank 0)
#define ENC28J60_CRE_ERXRDPTL 0x0C  // ERXRDPTL (Bank 0) RX Read Datapointer
#define ENC28J60_CRE_ERXRDPTH 0x0D  // ERXRDPTH (Bank 0)
#define ENC28J60_CRE_ERXWRPTL 0x0E  // ERXWRPTL (Bank 0) RX Write Datapointer
#define ENC28J60_CRE_ERXWRPTH 0x0F  // ERXWRPTH (Bank 0)

#define ENC28J60_CRM_MACON1   0x00  // MACON1   (Bank 2)
#define ENC28J60_CRM_MACON2   0x01  // MACON2   (Bank 2)
#define ENC28J60_CRM_MACON3   0x02  // MACON3   (Bank 2)
#define ENC28J60_CRM_MABBIPG  0x04  // MABBIPG  (Bank 2)
#define ENC28J60_CRM_MAIPGL   0x06  // MAIPGL   (Bank 2)
#define ENC28J60_CRM_MAMXFLL  0x0A  // MAMXFLL  (Bank 2)
#define ENC28J60_CRM_MAMXFLH  0x0B  // MAMXFLH  (Bank 2)
#define ENC28J60_CRM_MICMD    0x12  // MICMD    (Bank 2)
#define ENC28J60_CRM_MIREGADR 0x14  // MIREGADR (Bank 2)
#define ENC28J60_CRM_MIWRL    0x16  // MIWRL    (Bank 2)
#define ENC28J60_CRM_MIWRH    0x17  // MIWRH    (Bank 2)
#define ENC28J60_CRM_MIRDL    0x18  // MIRDL    (Bank 2)
#define ENC28J60_CRM_MIRDH    0x19  // MIRDH    (Bank 2)

#define ENC28J60_CRM_MAADR5   0x00  // MAADR5   (Bank 3) MAC-04 (MAC 00:01:02:03:04:05)
#define ENC28J60_CRM_MAADR6   0x01  // MAADR6   (Bank 3) MAC-05
#define ENC28J60_CRM_MAADR3   0x02  // MAADR3   (Bank 3) MAC-02
#define ENC28J60_CRM_MAADR4   0x03  // MAADR4   (Bank 3) MAC-03
#define ENC28J60_CRM_MAADR1   0x04  // MAADR1   (Bank 3) MAC-00
#define ENC28J60_CRM_MAADR2   0x05  // MAADR2   (Bank 3) MAC-01
#define ENC28J60_CRM_MISTAT   0x0A  // MISTAT   (Bank 3)
#define ENC28J60_CRE_EREVID   0x12  // EREVID   (Bank 3) IC-Versions-Nr

#define ENC28J60_CRE_EIE      0x1B  // EIE      (alle Baenke) Interrupt Enable
#define ENC28J60_CRE_EIR      0x1C  // EIR      (alle Baenke) Interrupt Flags
#define ENC28J60_CRE_ECON2    0x1E  // ECON2    (alle Baenke)
#define ENC28J60_CRE_ECON1    0x1F  // ECON1    (alle Baenke)

#define ENC28J60_CR_MAX_ADR  0x1F  // max adresse der Control-Register
#define ENC28J60_CR_MAX_BANK 0x03  // max bank-Nr im Control-Register


//-----------------------------------------------------------------------------
// interne Defines  PHY-Register
//-----------------------------------------------------------------------------
#define ENC28J60_PHY_PHCON1  0x00    // PHCON1
#define ENC28J60_PHY_PHID1   0x02    // PHID1 (IC-ID-Nr1)
#define ENC28J60_PHY_PHID2   0x03    // PHID2 (IC-ID-Nr2)
#define ENC28J60_PHY_PHLCON  0x14    // PHLCON

#define ENC28J60_PHY_MAX_ADR 0x1F    // max adresse der PHY-Regsiter



//-----------------------------------------------------------------------------
// interne Globale Variabeln
//-----------------------------------------------------------------------------
unsigned int ENC_RX_POINTER;         // Zeiger auf den Start vom RX-Packet
unsigned int ENC_RX_START;           // Start vom RX-Puffer
unsigned int ENC_RX_ENDE;            // Ende vom RX-Puffer
unsigned int ENC_TX_START;           // Start vom TX-Puffer
unsigned int ENC_TX_ENDE;            // Ende vom TX-Puffer



//-----------------------------------------------------------------------------
unsigned char ENC_readControlRegisterE(unsigned char adr) {
  // lesen vom Control Register
  // von aktueller Bank
  // WICHTIG !! nur fuer ETH-Register
  // siehe Datasheet Seite 27

  unsigned char ret_wert=0;

  // auf Maxwert pruefen
  if(adr>ENC28J60_CR_MAX_ADR) adr=0x00;

  MK3_PORT_ETH&=~ENC28J60_CS;  // Chip-Select EIN

  // RCR-Opcode hinzufuegen
  adr=adr|0x00;

  // zuerst die Adresse senden
  MK3_SPI_SEND(adr);

  // dann den Inhalt auslesen
  ret_wert=MK3_SPI_SEND(0x55);  // 0x55 = dummy kommando

  MK3_PORT_ETH|=ENC28J60_CS;   // Chip-Select AUS

  return (ret_wert);
}

//-----------------------------------------------------------------------------
unsigned char ENC_readControlRegisterM(unsigned char adr) {
  // lesen vom Control Register
  // von aktueller Bank
  // WICHTIG !! nur fuer MAC und MII-Register
  // siehe Datasheet Seite 27

  unsigned char ret_wert=0;

  // auf Maxwert pruefen
  if(adr>ENC28J60_CR_MAX_ADR) adr=0x00;

  MK3_PORT_ETH&=~ENC28J60_CS;  // Chip-Select EIN

  // RCR-Opcode hinzufuegen
  adr=adr|0x00;

  // zuerst die Adresse senden
  MK3_SPI_SEND(adr);

  // jetzt ein Dummy Byte senden
  MK3_SPI_SEND(0x55);  // 0x55 = dummy kommando

  // dann den Inhalt auslesen
  ret_wert=MK3_SPI_SEND(0x55);  // 0x55 = dummy kommando

  MK3_PORT_ETH|=ENC28J60_CS;   // Chip-Select AUS

  return (ret_wert);
}

//-----------------------------------------------------------------------------
void ENC_writeControlRegister(unsigned char adr,unsigned char wert) {
  // schreiben ins Control Register
  // in aktuelle Bank
  // fuer ETH, MAC, MII-Register
  // siehe Datasheet Seite 28

  // auf Maxwert pruefen
  if(adr>ENC28J60_CR_MAX_ADR) adr=0x00;

  MK3_PORT_ETH&=~ENC28J60_CS;  // Chip-Select EIN


  // WCR-Opcode hinzufuegen
  adr=adr|0x40;

  // zuerst die Adresse senden
  MK3_SPI_SEND(adr); 

  // dann die Daten senden
  MK3_SPI_SEND(wert);

  MK3_PORT_ETH|=ENC28J60_CS;   // Chip-Select AUS
}


//-----------------------------------------------------------------------------
void ENC_setControlRegisterBank(unsigned char bank) {
  // wechselt die Control-Register Bank
  // siehe Datasheet Seite 12
  unsigned char wert=0;

  // auf Maxwert pruefen
  if(bank>ENC28J60_CR_MAX_BANK) bank=0x00;

  // aktuelle Bank auslesen
  wert=ENC_readControlRegisterE(ENC28J60_CRE_ECON1);
  if(bank==0) {
    if((wert&0x03)!=0x00) {
      // richtige Bank setzen
      wert&=~0x01;
      wert&=~0x02;
      // neue Bank schreiben
      ENC_writeControlRegister(ENC28J60_CRE_ECON1,wert);
    }
  }
  else if(bank==1) {
    if((wert&0x03)!=0x01) {
      // richtige Bank setzen
      wert|=0x01;
      wert&=~0x02;
      // neue Bank schreiben
      ENC_writeControlRegister(ENC28J60_CRE_ECON1,wert);
    }
  }
  else if(bank==2) {
    if((wert&0x03)!=0x02) {
      // richtige Bank setzen
      wert&=~0x01;
      wert|=0x02;
      // neue Bank schreiben
      ENC_writeControlRegister(ENC28J60_CRE_ECON1,wert);
    }
  }
  else {
    if((wert&0x03)!=0x03) {
      // richtige Bank setzen
      wert|=0x01;
      wert|=0x02;
      // neue Bank schreiben
      ENC_writeControlRegister(ENC28J60_CRE_ECON1,wert);
    }
  }
}

//-----------------------------------------------------------------------------
void ENC_clearBitRegister(unsigned char adr,unsigned char wert) {
  // loeschen eines Bits vom Control Register
  // in aktuelle Bank
  // z.B. wert=0x81 -> Bit7 und Bit0 werden auf Lo gesetzt
  // siehe Datasheet Seite 29
  // Vorsicht : Funktioniert nur mit ETH-Registern

  // auf Maxwert pruefen
  if(adr>ENC28J60_CR_MAX_ADR) adr=0x00;

  MK3_PORT_ETH&=~ENC28J60_CS;  // Chip-Select EIN

  // BFC-Opcode hinzufuegen
  adr=adr|0xA0;

  // zuerst die Adresse senden
  MK3_SPI_SEND(adr); 

  // dann die Daten senden
  MK3_SPI_SEND(wert);

  MK3_PORT_ETH|=ENC28J60_CS;   // Chip-Select AUS
}

//-----------------------------------------------------------------------------
void ENC_setBitRegister(unsigned char adr,unsigned char wert) {
  // setzen eines Bits vom Control Register
  // in aktuelle Bank
  // z.B. wert=0x81 -> Bit7 und Bit0 werden auf Hi gesetzt
  // siehe Datasheet Seite 29
  // Vorsicht : Funktioniert nur mit ETH-Registern

  // auf Maxwert pruefen
  if(adr>ENC28J60_CR_MAX_ADR) adr=0x00;

  MK3_PORT_ETH&=~ENC28J60_CS;  // Chip-Select EIN

  // BFS-Opcode hinzufuegen
  adr=adr|0x80;

  // zuerst die Adresse senden
  MK3_SPI_SEND(adr); 

  // dann die Daten senden
  MK3_SPI_SEND(wert);

  MK3_PORT_ETH|=ENC28J60_CS;   // Chip-Select AUS
}

//-----------------------------------------------------------------------------
unsigned int ENC_readPhyRegister(unsigned char adr) {
  // lesen vom PHY Register
  // siehe Datasheet Seite 19
  unsigned int ret_wert=0;
  unsigned char wert;
  unsigned char n;
  unsigned char lo,hi;

  // auf Maxwert pruefen
  if(adr>ENC28J60_PHY_MAX_ADR) adr=0x00;

  // Adress in MIREGADR,Bank2 speichern
  ENC_setControlRegisterBank(2);
  ENC_writeControlRegister(ENC28J60_CRM_MIREGADR,adr);
  
  // MIIRD BIT in MICMD,Bank2 setzen
  wert=ENC_readControlRegisterM(ENC28J60_CRM_MICMD);
  wert|=0x01; // MIIRD-BIT auf Hi setzen
  ENC_writeControlRegister(ENC28J60_CRM_MICMD,wert);

  // warte bis BUSY-Flag geloescht ist
  ENC_setControlRegisterBank(3);
  for(n=0;n<50;n++) {
    // BUSY Bit in MISTAT,Bank3 auslesen
    wert=ENC_readControlRegisterM(ENC28J60_CRM_MISTAT);
    if((wert&0x01)==0) { // BUSY-BIT auswerten
      break; // schleife verlassen
    }
  }
  
  // MIIRD BIT in MICMD,Bank2 loeschen
  ENC_setControlRegisterBank(2);
  wert=ENC_readControlRegisterM(ENC28J60_CRM_MICMD);
  wert&=~0x01; // MIIRD-BIT auf Lo setzen
  ENC_writeControlRegister(ENC28J60_CRM_MICMD,wert);
  
  // MIRDL,Bank2 lesen
  lo=ENC_readControlRegisterM(ENC28J60_CRM_MIRDL);
  // MIRDH,Bank2 lesen
  hi=ENC_readControlRegisterM(ENC28J60_CRM_MIRDH);

  // 16bit wert ausrechnen
  ret_wert=(hi<<8);
  ret_wert=(ret_wert|lo);

  return (ret_wert);
}

//-----------------------------------------------------------------------------
void ENC_writePhyRegister(unsigned char adr, unsigned int wert) {
  // schreiben ins PHY Register
  // siehe Datasheet Seite 19
  unsigned char lo,hi;
  unsigned char n;

  // auf Maxwert pruefen
  if(adr>ENC28J60_PHY_MAX_ADR) adr=0x00;

  // Adress in MIREGADR,Bank2 speichern
  ENC_setControlRegisterBank(2);
  ENC_writeControlRegister(ENC28J60_CRM_MIREGADR,adr);

  // in 8Bit umformen
  lo=(wert&0x00FF);
  hi=(wert>>8);

  // Lo-Byte in MIWRL,Bank2 speichern
  ENC_writeControlRegister(ENC28J60_CRM_MIWRL,lo);

  // Hi-Byte in MIWRH,Bank2 speichern
  ENC_writeControlRegister(ENC28J60_CRM_MIWRH,hi);

  // warte bis BUSY-Flag geloescht ist
  ENC_setControlRegisterBank(3);
  for(n=0;n<50;n++) {
    // BUSY Bit in MISTAT,Bank3 auslesen
    wert=ENC_readControlRegisterM(ENC28J60_CRM_MISTAT);
    if((wert&0x01)==0) { // BUSY-BIT auswerten
      break; // schleife verlassen
    }
  }
}

//-----------------------------------------------------------------------------
unsigned char ENC_readBufferMemory(void) {
  // lesen vom Buffer Memory
  // von aktueller ReadPointer Adresse
  // siehe Datasheet Seite 28
  unsigned char ret_wert=0;

  MK3_PORT_ETH&=~ENC28J60_CS;  // Chip-Select EIN

  // RBM Code senden
  MK3_SPI_SEND(0x3A);

  // dann den Inhalt auslesen
  ret_wert=MK3_SPI_SEND(0x55);  // 0x55 = dummy kommando

  MK3_PORT_ETH|=ENC28J60_CS;   // Chip-Select AUS

  return(ret_wert);
}

//-----------------------------------------------------------------------------
void ENC_readBufferBlock(unsigned char mode, unsigned int laenge) {
  // lesen von einem ganzen Block
  // aus dem Buffer Memory
  // je nach mode
  // 0=InitwWerte (3 mal 2Bytes)
  // 1=Header (0 bis laenge)
  // 2=Footer (laenge bis RX_ETH_Data.frame_len)

  unsigned char lo,hi;
  unsigned int uint;

  if(mode==0) {
    // Initwerte auslesen

    // lesen START
    MK3_PORT_ETH&=~ENC28J60_CS;  // Chip-Select EIN

    // RBM Code senden
    MK3_SPI_SEND(0x3A);

    // Start Adresse vom naechsten Packet auslesen
    lo=MK3_SPI_SEND(0x55);  // 0x55 = dummy kommando
    hi=MK3_SPI_SEND(0x55);  // 0x55 = dummy kommando
    uint=(hi<<8);
    uint=(uint|lo);
    ENC_RX_POINTER=uint;

    // packetlaenge auslesen
    lo=MK3_SPI_SEND(0x55);  // 0x55 = dummy kommando
    hi=MK3_SPI_SEND(0x55);  // 0x55 = dummy kommando
    uint=(hi<<8);
    uint=(uint|lo);    
    RX_ETH_Data.eth_frame_len=uint;

    // receive status auslesen
    lo=MK3_SPI_SEND(0x55);  // 0x55 = dummy kommando
    hi=MK3_SPI_SEND(0x55);  // 0x55 = dummy kommando
    uint=(hi<<8);
    uint=(uint|lo);
    RX_ETH_Data.status=uint;

    // lesen STOP
    MK3_PORT_ETH|=ENC28J60_CS;   // Chip-Select AUS
  }
  else if(mode==1) {
    // header auslesen

    // lesen START
    MK3_PORT_ETH&=~ENC28J60_CS;  // Chip-Select EIN

    // RBM Code senden
    MK3_SPI_SEND(0x3A);

    // kompletten ETH-Header aus dem Puffer auslesen
    for(uint=0;uint<laenge;uint++) {
      RX_ETH_Data.raw_data[uint]=MK3_SPI_SEND(0x55);  // 0x55 = dummy kommando
    }

    // lesen STOP
    MK3_PORT_ETH|=ENC28J60_CS;   // Chip-Select AUS
  }
  else if(mode==2) {
    // footer auslesen

    // lesen START
    MK3_PORT_ETH&=~ENC28J60_CS;  // Chip-Select EIN

    // RBM Code senden
    MK3_SPI_SEND(0x3A);

    // den Rest des ETH-Protokolls aus dem Puffer auslesen
    for(uint=laenge;uint<RX_ETH_Data.frame_len;uint++) {
      RX_ETH_Data.raw_data[uint]=MK3_SPI_SEND(0x55);  // 0x55 = dummy kommando
    }

    // lesen STOP
    MK3_PORT_ETH|=ENC28J60_CS;   // Chip-Select AUS
  }
}

//-----------------------------------------------------------------------------
void ENC_writeBufferMemory(unsigned char wert) {
  // schreiben in den Buffer Memory
  // in aktuelle WritePointer Adresse
  // siehe Datasheet Seite 29

  MK3_PORT_ETH&=~ENC28J60_CS;  // Chip-Select EIN

  // WBM Code senden
  MK3_SPI_SEND(0x7A);

  // dann den Wert senden
  MK3_SPI_SEND(wert);

  MK3_PORT_ETH|=ENC28J60_CS;   // Chip-Select AUS
}

//-----------------------------------------------------------------------------
void ENC_writeBufferBlock(unsigned int laenge) {
  // schreiben von einem ganzen Block
  // in den Buffer Memory

  unsigned int n;
  unsigned char wert;

  // schreiben START
  MK3_PORT_ETH&=~ENC28J60_CS;  // Chip-Select EIN

  // WBM Code senden
  MK3_SPI_SEND(0x7A);

  // Init wert senden
  MK3_SPI_SEND(0x00);

  // kompletten Buffer schreiben
  for(n=0;n<laenge;n++) {
    wert=RX_ETH_Data.raw_data[n];
    MK3_SPI_SEND(wert);
  }

  // schreiben STOP
  MK3_PORT_ETH|=ENC28J60_CS;   // Chip-Select AUS
}

//-----------------------------------------------------------------------------
bool ENC_checkID(void) {
  // test des ENC28J60
  // auslesen der PHID-Nummern
  // zum verifizieren ob das ENC28J60 OK ist
  // siehe Datasheet Seite 22 
  bool ret_wert=true;
  unsigned int uint=0;

  // PHID-1 auslesen, speichern und testen
  uint=ENC_readPhyRegister(ENC28J60_PHY_PHID1);
  ENC28J60.phid1=uint;
  if(uint!=ETH_ENC_PHID1_SOLL) ret_wert=false;

  // PHID-2 auslesen,speichern und testen
  uint=ENC_readPhyRegister(ENC28J60_PHY_PHID2);
  uint=(uint&0xFC00);
  ENC28J60.phid2=uint;
  if(uint!=ETH_ENC_PHID2_SOLL) ret_wert=false;

  return (ret_wert);
}

//-----------------------------------------------------------------------------
unsigned char ENC_checkVersion(void) {
  // auslesen der Version vom ENC28J60
  unsigned char ret_wert=0;

  // EREVID,Bank3 lesen
  ENC_setControlRegisterBank(3);
  ret_wert=ENC_readControlRegisterE(ENC28J60_CRE_EREVID);

  return(ret_wert);
}

//-----------------------------------------------------------------------------
void ENC_setLedMode(ETH_LED_MODE_t led_a, ETH_LED_MODE_t led_b) {
  // einstellen der Modes der LEDs
  unsigned int mode_a,mode_b;
  unsigned int uint;

  // LED-A einstellen
  mode_a=0x0400; // default = Link Status
  if(led_a==LED_MODE_LINK) mode_a=0x0400; // default
  if(led_a==LED_MODE_RX) mode_a=0x0200;
  if(led_a==LED_MODE_TX) mode_a=0x0100;
  if(led_a==LED_MODE_RXTX) mode_a=0x0700;
  if(led_a==LED_MODE_ERR) mode_a=0x0300;
  if(led_a==LED_MODE_OFF) mode_a=0x0900;
  if(led_a==LED_MODE_ON) mode_a=0x0800;
  if(led_a==LED_MODE_BLINK1) mode_a=0x0B00;
  if(led_a==LED_MODE_BLINK2) mode_a=0x0A00;

  // LED-B einstellen
  mode_b=0x0020; // default = RX aktivity
  if(led_b==LED_MODE_LINK) mode_b=0x0040;
  if(led_b==LED_MODE_RX) mode_b=0x0020; // default
  if(led_b==LED_MODE_TX) mode_b=0x0010;
  if(led_b==LED_MODE_RXTX) mode_b=0x0070;
  if(led_b==LED_MODE_ERR) mode_b=0x0030;
  if(led_b==LED_MODE_OFF) mode_b=0x0090;
  if(led_b==LED_MODE_ON) mode_b=0x0080;
  if(led_b==LED_MODE_BLINK1) mode_b=0x00B0;
  if(led_b==LED_MODE_BLINK2) mode_b=0x00A0;

  // alten Wert auslesen
  uint=ENC_readPhyRegister(ENC28J60_PHY_PHLCON);
  // alten werte auf 0 setzen
  uint=(uint&0xF00F);
  // neue werte setzen
  uint=(uint|mode_a);
  uint=(uint|mode_b);
  // neuen Wert schreiben
  ENC_writePhyRegister(ENC28J60_PHY_PHLCON,uint);
}

//-----------------------------------------------------------------------------
void ENC_setBuffer(ETH_RAM_t aufteilung) {
  // RAM Aufteilung einstellen
  // komplettes RAM hat 8kByte = 0x0000 bis 0x1FFF

  //-----------------------------------------------
  // VORSICHT !! das hier ist ein Workaround
  // aus dem Errata-Sheet Punkt.5 (Seite 3) 
  // -> RX-Bereich muss im Lower-Segment von Puffer stehen
  //    TX-Bereich muss im Higher-Segment stehen
  //-----------------------------------------------

  unsigned char lo,hi;

  //-----------------------------
  ENC_RX_START=0x0000;
  ENC_RX_ENDE =0x0FFF;
  ENC_TX_START=0x1000;
  ENC_TX_ENDE =0x1FFF;
  
  //-----------------------------  
  if(aufteilung==RX_25_TX_75) ENC_TX_START=0x0800; // 25:75
  if(aufteilung==RX_50_TX_50) ENC_TX_START=0x1000; // 50:50
  if(aufteilung==RX_75_TX_25) ENC_TX_START=0x1800; // 75:25

  // RX-Ende setzen
  ENC_RX_ENDE=ENC_TX_START-1;

  ENC_setControlRegisterBank(0);
  // RX-Buffer Start schreiben
  lo=(ENC_RX_START&0x00FF);
  hi=(ENC_RX_START>>8);
  ENC_writeControlRegister(ENC28J60_CRE_ERXSTL,lo);  
  ENC_writeControlRegister(ENC28J60_CRE_ERXSTH,hi);
  // RX-Read Pointer schreiben
  ENC_writeControlRegister(ENC28J60_CRE_ERXRDPTL,lo);  
  ENC_writeControlRegister(ENC28J60_CRE_ERXRDPTH,hi);
  // RX-Buffer Ende schreiben
  lo=(ENC_RX_ENDE&0x00FF);
  hi=(ENC_RX_ENDE>>8);
  ENC_writeControlRegister(ENC28J60_CRE_ERXNDL,lo); 
  ENC_writeControlRegister(ENC28J60_CRE_ERXNDH,hi);

  // TX-Buffer Start schreiben
  lo=(ENC_TX_START&0x00FF);
  hi=(ENC_TX_START>>8);
  ENC_writeControlRegister(ENC28J60_CRE_ETXSTL,lo);  
  ENC_writeControlRegister(ENC28J60_CRE_ETXSTH,hi);
  // TX-Buffer Ende schreiben
  lo=(ENC_TX_ENDE&0x00FF);
  hi=(ENC_TX_ENDE>>8);
  ENC_writeControlRegister(ENC28J60_CRE_ETXNDL,lo);  
  ENC_writeControlRegister(ENC28J60_CRE_ETXNDH,hi);

  // Pointer aktuallisieren
  ENC_RX_POINTER=ENC_RX_START;

  // freien Speicher setzen
  ENC28J60.rx_free=(ENC_RX_ENDE-ENC_RX_START);
}

//-----------------------------------------------------------------------------
void ENC_softReset(void) {
  // System Reset
  // siehe Datasheet Seite 30

  MK3_PORT_ETH&=~ENC28J60_CS;  // Chip-Select EIN

  // SRC Code senden
  MK3_SPI_SEND(0xFF); 

  MK3_PORT_ETH|=ENC28J60_CS;   // Chip-Select AUS
}

//-----------------------------------------------------------------------------
void ENC_setMAC(void) {
  // MAC-Adresse einstellen
  //
  // im ENC28J60 wird die MAC-Adresse
  // nicht der Reihe nach gespeichert !!

  // MAADR5,Bank3 beschreiben
  ENC_setControlRegisterBank(3);
  ENC_writeControlRegister(ENC28J60_CRM_MAADR1,ETH_Setting.own_mac[0]); // adr 04
  ENC_writeControlRegister(ENC28J60_CRM_MAADR2,ETH_Setting.own_mac[1]); // adr 05
  ENC_writeControlRegister(ENC28J60_CRM_MAADR3,ETH_Setting.own_mac[2]); // adr 02
  ENC_writeControlRegister(ENC28J60_CRM_MAADR4,ETH_Setting.own_mac[3]); // adr 03
  ENC_writeControlRegister(ENC28J60_CRM_MAADR5,ETH_Setting.own_mac[4]); // adr 00
  ENC_writeControlRegister(ENC28J60_CRM_MAADR6,ETH_Setting.own_mac[5]); // adr 01
}

//-----------------------------------------------------------------------------
void ENC_initENC28J60(void) {
  // init vom ENC28J60
  // im FullDuplex-Mode

  unsigned char wert;
  unsigned int uint;
  unsigned char lo,hi;

  //----------------------------
  // MAC initialisieren (siehe Datasheet Seite 34)
  //----------------------------

  // MACON1,Bank2 beschreiben
  ENC_setControlRegisterBank(2);
  wert=ENC_readControlRegisterM(ENC28J60_CRM_MACON1);
  wert|=0x01; // MARXEN Bit setzen
  wert|=0x04; // RXPAUS Bit setzen
  wert|=0x08; // TXPAUS Bit setzen
  ENC_writeControlRegister(ENC28J60_CRM_MACON1,wert);  

  // MACON3,Bank2 beschreiben
  wert=ENC_readControlRegisterM(ENC28J60_CRM_MACON3);
  wert|=0x20; // PADCFG Mode auf "001" setzen
  wert|=0x10; // TXCRCEN Bit setzen
  wert|=0x02; // FRMLNEN Bit setzen
  wert|=0x01; // FULDPX Bit setzen
  ENC_writeControlRegister(ENC28J60_CRM_MACON3,wert);

  // MAMXFL,Bank2 beschreiben
  uint=1518; // Max-Frame groesse
  lo=(uint&0x00FF);
  hi=(uint>>8); 
  ENC_writeControlRegister(ENC28J60_CRM_MAMXFLL,lo); 
  ENC_writeControlRegister(ENC28J60_CRM_MAMXFLH,hi);

  // MABBIPG,Bank2 beschreiben
  wert=0x15; // fuer Full-Duplex
  ENC_writeControlRegister(ENC28J60_CRM_MABBIPG,wert);

  // MAIPGL,Bank2 beschreiben
  wert=0x12; 
  ENC_writeControlRegister(ENC28J60_CRM_MAIPGL,wert);

  //----------------------------
  // PHY initialisieren (siehe Datasheet Seite 37)
  //----------------------------
  
  // PHCON1 auslesen und check ob fullduplex
  uint=ENC_readPhyRegister(ENC28J60_PHY_PHCON1);
  if((uint&0x0100)==0) {
    // HalbDuplex-Mode
    //
    // umschalten auf FullDuplex-Mode
    uint|=0x0100;
    ENC_writePhyRegister(ENC28J60_PHY_PHCON1,uint);
  }
}

//-----------------------------------------------------------------------------
void ENC_rxEnable(void) {
  // enable der RX-Funktion

  unsigned char wert=0;

  wert|=0x04; // RXEN Bit setzen
  ENC_setBitRegister(ENC28J60_CRE_ECON1,wert);
}

//-----------------------------------------------------------------------------
void ENC_IntEnable(void) {
  // enable der Interrupts

  unsigned char wert=0;

  wert=0x00;
  wert|=0x40; // PKTIF Bit loeschen
  wert|=0x01; // RXERIF Bit loeschen
  ENC_clearBitRegister(ENC28J60_CRE_EIR,wert);

  wert=0x00;
  wert|=0x80; // INTIE Bit setzen
  wert|=0x40; // PKTIE Bit setzen
  wert|=0x01; // RXERIE Bit setzen
  ENC_setBitRegister(ENC28J60_CRE_EIE,wert);
}

//-----------------------------------------------------------------------------
unsigned int ENC_getFreeRXPuffer(void) {
  // ausrechnen des noch freien RX-Puffers
  // siehe Datasheet Seite 45

  unsigned int ret_wert=0;
  unsigned char lo,hi;
  unsigned int uint;
  unsigned int v_rx_wr_ptr=0;
  unsigned int v_rx_rd_ptr=0;

  // ERXWRPT,Bank0 auslesen
  ENC_setControlRegisterBank(0);
  lo=ENC_readControlRegisterE(ENC28J60_CRE_ERXWRPTL);
  hi=ENC_readControlRegisterE(ENC28J60_CRE_ERXWRPTH);
  uint=(hi<<8);
  uint=(uint|lo);
  v_rx_wr_ptr=uint;

  // ERXRDPT,Bank0 auslesen
  lo=ENC_readControlRegisterE(ENC28J60_CRE_ERXRDPTL);
  hi=ENC_readControlRegisterE(ENC28J60_CRE_ERXRDPTH);
  uint=(hi<<8);
  uint=(uint|lo);
  v_rx_rd_ptr=uint;  

  // ausrechnen des noch freien RX-Speichers 
  if(v_rx_wr_ptr>v_rx_rd_ptr) {
    ret_wert=(ENC_RX_ENDE-ENC_RX_START)-(v_rx_wr_ptr-v_rx_rd_ptr);
  }
  else if(v_rx_wr_ptr==v_rx_rd_ptr) {
    ret_wert=(ENC_RX_ENDE-ENC_RX_START);
  }
  else {
    ret_wert=(v_rx_rd_ptr-v_rx_wr_ptr)-1;
  }

  return(ret_wert);
}


#endif  // Ende von #define UB_ETH_ENC


