//-----------------------------------------------------------------------------
// ETH_LO_GLOBAL
// Version 3.0
// 17.12.2011 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Lo-Level Funktionen
// mit dem Ethernet-IC : ENC28J60
//
// Funktionen : Globale Hardware-Settings und Enumerationen
//              hier kann der Defaultwert der MAC und IP-Adresse
//              und andere Settings hinterlegt werden
//
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_EXT_MODULE/UB_ETH_LOLEVEL/"
// liegen und mit #include <UB_ETH_LOLEVEL/ETH_LO_GLOBAL.h>
// eingebunden werden
//-----------------------------------------------------------------------------


#ifndef UB_ETH_GLOBAL    // Wichtig !!
#define UB_ETH_GLOBAL 1  // damit wird das File nur einmal eingebunden


//----------------------------------------------------------------------------
// Globale Einstellung :
// Interrupt oder Polling Betrieb
// falls der Interrupt-Pin (PB5) nicht benutzt werden soll,
// einfach den Define auskommentieren
// dann muss die Funktion "EXT_ETH_POLL" zyklisch aufgerufen werden
//----------------------------------------------------------------------------
#define USE_ETH_INTMODE 1   // 1=Interrupt-Bertrieb, 0=Polling-Betrieb


//----------------------------------------------------------------------------
// Globale Einstellung :
// Hardware oder Software Reset
// falls der Hardware-Reset-Pin (PB6) nicht benutzt werden soll,
// einfach den Define auskommentieren
//----------------------------------------------------------------------------
#define USE_ETH_HWRESET 1   // 1=Hardware Reset, 0=Software Reset



//----------------------------------------------------------------------------
// Globale Einstellung :
// Freien Empfangs-Speicher nach jeden Packet_Receive berechnen ?
// falls whrend der Laufzeit der noch freie RX-Puffer
// ausgelesen werden soll
//----------------------------------------------------------------------------
#define USE_ETH_CALC_RX_FREE 0   // 1=freien Speicher berechnen, 0=nicht berechnen




//-----------------------------------------------------------------------------
// Port-Pin definition
//-----------------------------------------------------------------------------
#define MK3_DDR_ETH DDRB        // DDR-Register der ETH-Signale
#define MK3_PORT_ETH PORTB      // Port der ETH-Signale
#define MK3_PIN_ETH PINB        // Pin der ETH-Signale (fuer Interrupt) 

#define ENC28J60_CS  0x10       // ChipSelect = Bit4
#define ENC28J60_INT 0x20       // Interrupt  = Bit5
#define ENC28J60_RESET 0x40     // Reset  = Bit6


//-----------------------------------------------------------------------------
// Enumerationen fuer alle Funktionen
//-----------------------------------------------------------------------------
typedef enum {
  LED_MODE_DEFAULT=0,// stellt den Defaultwert ein
  LED_MODE_LINK,     // Zeigt den Link-Status an (default fuer LED-A)
  LED_MODE_RX,       // Zeigt nur RX aktivity (default fuer LED-B)
  LED_MODE_TX,       // Zeigt nur TX aktivity
  LED_MODE_RXTX,     // Zeigt RX+TX aktivity
  LED_MODE_ERR,      // Zeigt collision aktivity
  LED_MODE_OFF,      // LED immer AUS
  LED_MODE_ON,       // LED immer EIN
  LED_MODE_BLINK1,   // LED blinkt immer (langsam)
  LED_MODE_BLINK2    // LED blinkt immer (schnell)
}ETH_LED_MODE_t;

typedef enum {
  RX_50_TX_50 =0,    // stellt RX und TX Buffer auf je 4kByte
  RX_25_TX_75,       // stellt RX auf 2k und TX auf 6kByte
  RX_75_TX_25        // stellt RX auf 6k und TX auf 2kByte
}ETH_RAM_t;

typedef enum {
  ETH_TYP_UNKNOWN =0, // unbekanntes Typfield -> Packet verwerfen
  ETH_TYP_ARP,        // ARP Packet
  ETH_TYP_IP          // IP-Packet
}ETH_TYP_t;

typedef enum {
  ETH_RX_NO_PACKET =0,    // kein Packet emfangen
  ETH_RX_SIZE_ERR,        // wenn die Packet laenge zu gross war
  ETH_RX_STAT_ERR,        // Fehler im Status-Vector
  ETH_RX_UNKNOWN,         // wenn das Packet nicht unterstuetzt wird
  ETH_RX_NOT_FOR_ME,      // wenn das Packet nicht fuer mich war
  ETH_RX_OK               // wenn das Packet OK war
}ETH_RX_STAT_t;

typedef enum {
  ETH_ARP_UNKNWON =0,     // ARP-Typ unbekannt
  ETH_ARP_REQUEST,        // ARP-Typ = Anfrage
  ETH_ARP_REPLY           // ARP-Typ = Antwort
}ETH_ARP_TYP_t;

typedef enum {
  ETH_IP_UNKNOWN =0,      // IP-Typ unbekannt
  ETH_IP_ICMP,            // IP-TYP = ICMP (Ping)
  ETH_IP_UDP              // IP-Typ = UDP
}ETH_IP_TYP_t;

typedef enum {
  ETH_ICMP_UNKNWON =0,    // ICMP-Typ unbekannt
  ETH_ICMP_REQUEST,       // ECHO-Typ = Anfrage
  ETH_ICMP_REPLY          // ECHO-Typ = Antwort
}ETH_ICMP_TYP_t;

typedef enum {
  ETH_TX_NOTHING =0,      // keine Antwort notwendig
  ETH_TX_ARP_REPLY,       // ARP-Reply soll gesendet werden
  ETH_TX_ICMP_REPLY,      // ICMP-Reply soll gesendet werden
  ETH_TX_UDP_PACKET       // UDP-Packet soll gesendet werden
}ETH_TX_TYP_t;


//-----------------------------------------------------------------------------
// Globale Einstellungen :
// Definition der Groesse fuer die
// verschiedenen Protokolle
//-----------------------------------------------------------------------------

#define  ETH_MAX_PACKET_SIZE  200   // max groesse eines packets (fuer RX und TX)
#define  ETH_MAX_ARP_SIZE     100   // max size ETH-Frame bei ARP (normal=64)
#define  ETH_MAX_ICMP_SIZE    100   // max size ETH-Frame bei ICMP (normal=78)
#define  ETH_MAX_UDP_SIZE     100   // max size ETH-Frame bei UDP (min=64)


//-----------------------------------------------------------------------------
// Struktur der Ethernet Settings
// kann vom User vor dem Aufruf von "EXT_ETH_START()"
// angepasst werden
//-----------------------------------------------------------------------------
typedef struct {
  unsigned char own_mac[6];  // eigene MAC-Adresse
  unsigned char own_ip[4];   // eigene IP-Adresse
  unsigned int own_udp_port; // eigener UDP-Port
  ETH_LED_MODE_t led_a;      // Mode der LED-A
  ETH_LED_MODE_t led_b;      // Mode der LED-B
  ETH_RAM_t buffer;          // Puffer Aufteilung
}ETH_Setting_t;
ETH_Setting_t ETH_Setting;

//-----------------------------------------------------------------------------
// Struktur vom Ethernet IC
//-----------------------------------------------------------------------------
typedef struct {
  bool status_ok;            // true, wenn IC beim Init gefunden wurde
  unsigned int phid1;        // istwert vom PHID1-Register
  unsigned int phid2;        // istwert vom PHID2-Register
  unsigned char version;     // Hardware-Version (bei UB : 0x06 = Version = B7)
  unsigned int rx_packete;   // Anzahl der empfangenen Packete (die ok waren)
  unsigned int rx_unknonwn;  // Anzahl der unbekannten Packete (die verworfen wurden)
  unsigned int rx_error;     // Anzahl der fehlerhaften Packete
  unsigned int rx_free;      // freier RX-speicher (in Bytes)
  unsigned int tx_packete;   // Anzahl der gesendeten Packete
}ENC28J60_t;
ENC28J60_t ENC28J60;         // Struktur des Ethernet ICs

//-----------------------------------------------------------------------------
// Struktur der Host Daten
//-----------------------------------------------------------------------------
typedef struct {
  bool host_ok;              // true, wenn alle Host-Daten bekannt sind
  unsigned char host_mac[6]; // HOST MAC-Adresse (aus MAC-Frame)
  unsigned char host_ip[4];  // HOST IP-Adresse
}HOST_ETH_Data_t;
HOST_ETH_Data_t HOST_ETH_Data;

//-----------------------------------------------------------------------------
// Struktur der RX-Packet Daten
//-----------------------------------------------------------------------------
typedef struct {
  unsigned int eth_frame_len;// laenge vom Ethernet Frame (in bytes)
  unsigned int status;       // status vom Ethernet-Frame
  unsigned char raw_data[ETH_MAX_PACKET_SIZE]; // roh daten
  unsigned int typfield;     // typfield vom Ethernet Frame (als Hex-Zahl)
  ETH_TYP_t ethtyp;          // typ vom Ethernet Frame
  unsigned int frame_len;    // laenge Frame (zum auslesen)
  unsigned char z_mac[6];    // Ziel MAC-Adresse  (aus MAC-Frame)
  unsigned int ip_frame_len; // laenge vom IP-Frame
  unsigned int ip_data_len;  // laenge vom IP-daten bereich
  unsigned int udp_q_port;   // quellport (bei UDP)
  unsigned int udp_z_port;   // zielport (bei UDP)
  unsigned int udp_data_len; // laenge der UDP-Daten (ohne header)
}RX_ETH_Data_t;
RX_ETH_Data_t RX_ETH_Data;

//-----------------------------------------------------------------------------
// Struktur der TX-Packet Daten
//-----------------------------------------------------------------------------
typedef struct {
  ETH_TX_TYP_t tx_typ;       // Typ der Antwort
  unsigned int ip_id_nr;     // Lfd-ID-Nummer
  unsigned int eth_tx_len;   // Laenge vom ETH-Frame (ohne Checksumme)
  unsigned int udp_data_len; // laenge der UDP-Daten (ohne header)
}TX_ETH_Data_t;
TX_ETH_Data_t TX_ETH_Data;



//-----------------------------------------------------------------------------
// Timeout fuer ARP-Request
// der per "default" benutzt wird
//-----------------------------------------------------------------------------
#define ETH_ARP_REQUEST_TIMEOUT_POLL  30     // 30   -> ca 100ms
#define ETH_ARP_REQUEST_TIMEOUT_INT   800    // 800  -> ca 100ms


//-----------------------------------------------------------------------------
// OUI-Nummer vom ENC28J60
// die per "default" benutzt wird
// muss u.U. an den eigenen ENC28J60 angepasst werden
// falls "ENC28J60.status_ok" immer mit FALSE getestet wird
//-----------------------------------------------------------------------------
#define ETH_ENC_PHID1_SOLL  0x0083  // Inhalt von PHID1-Register (16bit)
#define ETH_ENC_PHID2_SOLL  0x1400  // Inhalt von PHID2-Register (6bit)


//-----------------------------------------------------------------------------
// MAC-Adresse vom ENC28J60 [00:21:43:F7:3A:01]
// die per "default" benutzt wird
// kann vor dem Start ueberschrieben werden mit "ETH_Setting.own_mac"
//-----------------------------------------------------------------------------
#define DEF_ETH_MAC_0  0x00
#define DEF_ETH_MAC_1  0x21
#define DEF_ETH_MAC_2  0x43
#define DEF_ETH_MAC_3  0xF7
#define DEF_ETH_MAC_4  0x3A
#define DEF_ETH_MAC_5  0x01


//-----------------------------------------------------------------------------
// IP-Adresse vom ENC28J60 [192.168.115.110]
// die per "default" benutzt wird
// kann vor dem Start ueberschrieben werden mit "ETH_Setting.own_ip"
//-----------------------------------------------------------------------------
#define DEF_ETH_IP_0  192
#define DEF_ETH_IP_1  168
#define DEF_ETH_IP_2  115
#define DEF_ETH_IP_3  110


//-----------------------------------------------------------------------------
// IP-Adresse vom Ziel Host [192.168.115.1]
// die per "default" benutzt wird
// kann im Betrieb ueberschrieben werden mit "HOST_ETH_Data.host_ip"
//-----------------------------------------------------------------------------
#define DEF_HOST_IP_0  192
#define DEF_HOST_IP_1  168
#define DEF_HOST_IP_2  115
#define DEF_HOST_IP_3  1


//-----------------------------------------------------------------------------
// UDP-Port [65100]
// der per "default" benutzt wird
// kann im Betrieb ueberschrieben werden mit "ETH_Setting.own_udp_port"
//-----------------------------------------------------------------------------
#define DEF_ETH_UDP_PORT  65100


//-----------------------------------------------------------------------------
// LED-Mode
// der per "default" benutzt wird
// kann vor dem Start ueberschrieben werden
// mit "ETH_Setting.led_a" und "ETH_Setting.led_b"
//-----------------------------------------------------------------------------
#define DEF_ENC_LED_MODE_A   LED_MODE_LINK
#define DEF_ENC_LED_MODE_B   LED_MODE_RX


//-----------------------------------------------------------------------------
// RAM-Aufteiung
// die per "default" benutzt wird
// kann vor dem Start ueberschrieben werden mit "ETH_Setting.buffer"
//-----------------------------------------------------------------------------
#define DEF_ENC_BUFFER_MODE   RX_50_TX_50







#endif  // Ende von #define UB_ETH_GLOBAL

